@@ -47,12 +47,11 @@ class FailedOpeningDeviceException(Exception):
 
 class Descriptor(object):
 
-    def __init__(self, pid, version, fingerprint, serial=None):
+    def __init__(self, key_type, mode, version, fingerprint, serial=None):
         self._logger = logger.getChild('Descriptor')
         self._version = version
-        self._pid = pid
-        self._key_type = pid.get_type()
-        self._mode = Mode.from_pid(pid)
+        self._key_type = key_type
+        self._mode = mode
         self._fingerprint = fingerprint
 
     @property
@@ -63,10 +62,6 @@ class Descriptor(object):
     def version(self):
         return self._version
 
-    @property
-    def pid(self):
-        return self._pid
-
     @property
     def mode(self):
         return self._mode
@@ -81,21 +76,24 @@ class Descriptor(object):
 
         transports &= self.mode.transports
 
-        logger.debug('Opening driver for serial: %s, pid: %s', serial, self.pid)
+        logger.debug('Opening driver for serial: %s, type: %s, mode: %s',
+                     serial, self.key_type, self.mode)
         for attempt in range(1, attempts + 1):
             logger.debug('Attempt %d of %d', attempt, attempts)
             sleep_time = attempt * 0.1
             for drv in _list_drivers(transports):
-                logger.debug('Found driver: %s, pid: %s', drv, drv.pid)
+                logger.debug('Found driver: %s, key_type: %s, mode: %s',
+                             drv, drv.key_type, drv.mode)
                 dev = YubiKey(self, drv)
                 if serial is not None and dev.serial != serial:
                     logger.debug('Serial does not match. Want: %s, got: %s',
                                  serial, dev.serial)
                     del dev
                     continue
-                if drv.pid != self.pid:
-                    logger.debug('PID does not match. Want: %s, got: %s',
-                                 self.pid, drv.pid)
+                if (drv.key_type, drv.mode) != (self.key_type, self.mode):
+                    logger.debug('Descriptor mismatch. Want: %s, got: %s',
+                                 (self.key_type, self.mode),
+                                 (drv.key_type, drv.mode))
                     del dev
                     continue
                 return dev
@@ -111,12 +109,12 @@ class Descriptor(object):
         version = ((v_int >> 8) % 16, (v_int >> 4) % 16, v_int % 16)
         pid = PID(usb_dev.idProduct)
         fp = (pid, version, usb_dev.bus, usb_dev.address, usb_dev.iSerialNumber)
-        return cls(pid, version, fp)
+        return cls(pid.get_type(), Mode.from_pid(pid), version, fp)
 
     @classmethod
     def from_driver(cls, driver):
-        fp = (driver.pid,)
-        return cls(driver.pid, None, fp)
+        fp = (driver.key_type, driver.mode)
+        return cls(driver.key_type, driver.mode, None, fp)
 
 
 def _gen_descriptors():
@@ -155,36 +153,46 @@ def list_devices(transports=sum(TRANSPORT)):
         yield YubiKey(Descriptor.from_driver(d), d)
 
 
-def _open_driver(transports, serial, pid, attempts):
-    logger.debug('Opening driver for transports: %s, serial: %s, pid: %s',
-                 transports, serial, pid)
+def _open_driver(transports, serial, key_type, mode, attempts):
+    logger.debug('Opening driver for transports: %s, serial: %s, key_type: %s, '
+                 'mode: %s',
+                 transports, serial, key_type, mode)
     for attempt in range(1, attempts + 1):
         logger.debug('Attempt %d of %d', attempt, attempts)
         sleep_time = attempt * 0.1
         for dev in list_devices(transports):
-            logger.debug('Found driver: %s serial: %s, pid: %s',
-                         dev.driver, dev.serial, dev.driver.pid)
+            logger.debug('Found driver: %s serial: %s, key_type: %s, mode: %s',
+                         dev.driver, dev.serial, dev.driver.key_type,
+                         dev.driver.mode)
             if serial is not None and dev.serial != serial:
                 logger.debug('Serial does not match. Want: %s, got: %s',
                              serial, dev.serial)
                 del dev
                 continue
-            if pid is not None and dev.driver.pid != pid:
-                logger.debug('PID does not match. Want: %s, got: %s',
-                             pid, dev.driver.pid)
+            if key_type is not None and dev.driver.key_type != key_type:
+                logger.debug('Key type does not match. Want: %s, got: %s',
+                             key_type, dev.driver.key_type)
                 del dev
                 continue
             return dev.driver
+            if mode is not None and dev.driver.mode != mode:
+                logger.debug('Mode does not match. Want: %s, got: %s',
+                             mode, dev.driver.mode)
+                del dev
+                continue
         #  Wait a little before trying again.
         logger.debug('Sleeping for %f s', sleep_time)
         time.sleep(sleep_time)
-    logger.debug('No driver found for serial: %s, pid: %s', serial, pid)
+        logger.debug('No driver found for serial: %s, key_type: %s, mode: %s',
+                     serial, key_type, mode)
     raise FailedOpeningDeviceException()
 
 
-def open_device(transports=sum(TRANSPORT), serial=None, pid=None, attempts=10):
-    driver = _open_driver(transports, serial, pid, attempts)
-    matches = [d for d in get_descriptors() if d.pid == driver.pid]
+def open_device(transports=sum(TRANSPORT), serial=None, key_type=None,
+                mode=None, attempts=10):
+    driver = _open_driver(transports, serial, key_type, mode, attempts)
+    matches = [d for d in get_descriptors() if (d.key_type, d.mode)
+               == (driver.key_type, driver.mode)]
     if len(matches) == 1:  # Only one matching descriptor, must be it
         descriptor = matches[0]
     else:
